@@ -18,6 +18,11 @@ import platform
 import ctypes
 import time
 
+# Whether or not to tell the spincore library to write debug logfiles.
+# User can set to False before calling any spinapi functions to disable debugging.
+debug = False
+
+this_folder = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))
 def _checkloaded():
     global _spinapi
     try:    
@@ -28,13 +33,16 @@ def _checkloaded():
             libname = 'spinapi.dll'
         elif arch == ('64bit', 'WindowsPE'):
             libname = 'spinapi64.dll'
-        else:
-            raise NotImplementedError("No Unix support yet - testing this would require building shared objects from the spincore API on linux, and I don't know how to do this.")
-        _spinapi = ctypes.cdll.LoadLibrary(libname)
-        # enable debugging by default:
-        pb_set_debug(1)
+        elif arch == ('32bit', 'ELF'):
+            libname = os.path.join(this_folder, 'libspinapi.so')
+        elif arch == ('64bit', 'ELF'):
+            libname = os.path.join(this_folder, 'libspinapi64.so')
 
-# Defines for different pb_inst instruction types
+        _spinapi = ctypes.cdll.LoadLibrary(libname)
+        # enable debugging if it's switched on by the module global:
+        pb_set_debug(debug)
+
+# Defines for different pb_instr instruction types
 CONTINUE = 0
 STOP = 1
 LOOP = 2
@@ -84,7 +92,12 @@ def pb_set_debug(debug):
     _checkloaded()
     _spinapi.pb_set_debug.restype = ctypes.c_int
     return _spinapi.pb_set_debug(ctypes.c_int(debug))
-    
+
+def pb_get_version():
+    _checkloaded()
+    _spinapi.pb_get_version.restype = ctypes.c_char_p
+    return _spinapi.pb_get_version()
+
 def pb_get_error():
     _checkloaded()
     _spinapi.pb_get_error.restype = ctypes.c_char_p
@@ -211,21 +224,25 @@ def pb_inst_dds2(freq0,phase0,amp0,dds_en0,phase_reset0,
     return result
 
 # More convenience functions:
-def program_freq_regs(*freqs):
+def program_freq_regs(*freqs, **kwargs):
+    call_stop_programming = kwargs.pop('call_stop_programming', True)
     pb_start_programming(FREQ_REGS)
     for freq in freqs:
         pb_set_freq(freq)
-    pb_stop_programming()
+    if call_stop_programming:
+        pb_stop_programming()
     if len(freqs) == 1:
         return 0
     else:
         return tuple(range(len(freqs)))
 
-def program_phase_regs(*phases):
+def program_phase_regs(*phases, **kwargs):
+    call_stop_programming = kwargs.pop('call_stop_programming', True)
     pb_start_programming(PHASE_REGS)
     for phase in phases:
         pb_set_phase(phase)
-    pb_stop_programming()
+    if call_stop_programming:
+        pb_stop_programming()
     if len(phases) == 1:
         return 0
     else:
